/* Ack Applet

@Author: Christopher Hylands, based on Prime.java by Tom Parks

@Version:@(#)AckApplet.java	1.2 02/05/98

@Copyright (c) 1998 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION_2
                                                COPYRIGHTENDKEY
*/
import java.applet.Applet;
import java.awt.*;
import java.net.*;              // Need URL
import ptplot.*;

//////////////////////////////////////////////////////////////////////////
//// ackApplet
/** 
 */

public class AckApplet extends TextPlotApplet {

    public void addPoint(int dataset, double x, double y, boolean connected) {
        timeplot.addPoint(dataset, x, y, connected);
    }
    /** Return a string describing this applet.
     */
    public String getAppletInfo() {
        return "AckApplet: Applet interface to Kernighan & Wyk's " +
            "ack timing tests\n" +
            "By: Christopher Hylands, cxh@eecs.berkeley.edu\n" +
            "@(#)AckApplet.java	1.2 02/05/98";
    }

    public void init() {
        timeplot = new Plot();
        timeplot.setYLog(true);
        timeplot.setXRange(5,8);
        timeplot.setYRange(-3,1);
        timeplot.setTitle("Ackermann's Function ack(3,m)");
        timeplot.setNumSets(2);
        timeplot.setXLabel("m");
        timeplot.setYLabel("seconds");
        timeplot.setMarksStyle("various");
        timeplot.addLegend(0, "JavaScript");
        timeplot.addLegend(1, "Java Applet");
        super.init();
    }
    public void runTest () {
	_frame.setCursor(Frame.WAIT_CURSOR);
        int loops = 0;
	try
	{
	    loops = Integer.parseInt(_loopField.getText());
	} catch (NumberFormatException e) {}
        output.appendText("loop: "+loops);

        _connected = false;
        runOneTest(loops, 3, 5);
        runOneTest(loops, 3, 6);
        runOneTest(loops, 3, 7);
        runOneTest(loops, 3, 8);
	_frame.setCursor(Frame.DEFAULT_CURSOR);
    }

    public void runOneTest  (int loops, int n, int m) {
        String argv[] = new String[2];
        argv[0] = String.valueOf(n);
        argv[1] = String.valueOf(m);
        String results;
        for(int i = 0; i<loops; i++) {
            ack ack = new ack();
            results = ack.runack(argv);
            output.appendText("# " + results + "\n");
            // Get the middle field
            double time = Double.valueOf(results.substring(
                    results.indexOf(' ')+1,
                    results.lastIndexOf(' '))).doubleValue()/1000.0;
            timeplot.addPoint(1,m,time,_connected);
            if (_connected == false)
                _connected = true;
            output.appendText(m + " " + time + "\n");
        }
    }

    boolean _connected = false;
}
