// JavaScript versions of Kernighan and Wyk's timing tests.
//
// @Author: Christopher Hylands
// @Version: @(#)Ptplot.js	1.2 12/17/97
//
// @Copyright (c) 1997 The Regents of the University of California.
// All rights reserved.
// 
// Permission is hereby granted, without written agreement and without
// license or royalty fees, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in all
// copies of this software.
// 
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
// THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
// PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.
//
//                                                PT_COPYRIGHT_VERSION_2
//                                                COPYRIGHTENDKEY

//////////////////////////////////////////////////////////////////////////
// sumloop
//

function sumloop(n) {
    d0 = new Date();
    var sum = 0;
    for (var i=0; i < n; i++) {
	sum++;
    }
    d1 = new Date();
    dt = (d1.getTime() - d0.getTime())/1000.0;
    //alert(sum+" "+dt+"\n");
    //document.write(sum+" "+dt+"\n");
    return dt
}

function foo() {
}

function timesumloop (n) {
    document.write("<pre>\n");
    document.write("DataSet: JavaScript sumloop\n");
    for(i=125000; i <= 1000000; i = i*2) {
        for (j=0;j<n;j++) {
            r=sumloop(j);
            document.write(i + " " + r + "\n");
        }
    }
    document.write("</pre>\n");
}

function plotsumloop(loops) {
    n=loops.value;
    connected=false;
    document.Sumloop.output.appendText("Dataset: JavaScript sumloop\n");
    document.Sumloop.output.appendText("# "+n+": 125000 - 1000000\n");
    for(i=125000; i <= 1000000; i = i*2) {
        for(j=0;j<n;j++) {
            r=sumloop(i);
            document.Sumloop.addPoint(0,i,r,connected);
            document.Sumloop.output.appendText(i + " " + r + "\n");
            if (connected == false)
                connected=true;
        }
    }
    document.Sumloop.output.appendText("# Done with JavaScript sumloop\n");
}
//////////////////////////////////////////////////////////////////////////
// ackf
//
function ackf(m, n) {
    if (m == 0)
        return n+1;
    else if (n == 0)
        return ackf(m-1, 1);
    else
        return ackf(m-1, ackf(m, n-1));
}

function runack(m,n) {
    d0 = new Date();
    
    a = ackf(m, n);
    d1 = new Date();
    dt = (d1.getTime() - d0.getTime())/1000.0; 
    return dt;
}

function timeackf(n) {
    document.write("<pre>\n");
    document.write("DataSet: JavaScript ack\n");
    for (i=0;i<n;i++) {
        dt=runack(3,5);
        document.write(n + " " + dt + "\n");
    }
    document.write("# Not computing m=6, m=7 or m=8, JavaScript can't handle the recursion depth\n");

    //for (i=0;i<n;i++)
    //    runack(3,6);
    //for (i=0;i<n;i++)
    //    runack(3,7);
    //for (i=0;i<n;i++)
    //    runack(3,8);
    document.write("</pre>\n");

}

function plotackf(loops) {
    n=loops.value;
    connected=false;
    document.Ack.output.appendText("Dataset: JavaScript ack\n");
    m=5
    for(j=1;j<n;j++) {
        r=runack(3,m);
        document.Ack.addPoint(0,m,r,connected);
        document.Ack.output.appendText(m + " " + r + "\n");
        if (connected == false)
            connected=true;
    }
    if (n>=1) {
        r=runack(3,m);
        document.Ack.addPoint(0,m,r,false);
        document.Ack.output.appendText(m + " " + r + "\n");
    }
    document.Ack.output.appendText("# Done with JavaScript ack\n");
}


//////////////////////////////////////////////////////////////////////////
// array1
//
function runarray1(n) {
    ip = new Array(n);
    jp = new Array(n);
    d0 = new Date();

    for (var i = 0; i < n; i++)
        ip[i] = i ;
    for (var j = n-1; j >= 0; j--)
        jp[j] = ip[j];

    d1 = new Date();
    dt = (d1.getTime() - d0.getTime())/1000.0;
    document.write("#"+n+"\n");
    document.write(n+" "+dt+"\n");
}

function timearray1(n) {
    document.write("<pre>\n");
    document.write("DataSet: JavaScript array1\n");
    for (i=0;i<n;i++)
        runarray1(25000);
    for (i=0;i<n;i++)
        runarray1(50000);
    for (i=0;i<n;i++)
        runarray1(100000);
    for (i=0;i<n;i++)
        runarray1(200000);
    document.write("</pre>\n");
}

//////////////////////////////////////////////////////////////////////////
// string
//
function runstring(n) {
    d0 = new Date();

    for (j = 0; j < 10; j++) {
        s = "abcdef";
        while (s.length <= n) {
            s = "123" + s + "456" + s + "789";
            s = s.substring(s.length/2) + 
                s.substring(0, s.length/2+1);
        }
    }

    d1 = new Date();
    dt = (d1.getTime() - d0.getTime())/1000.0;
    document.write("#"+n+"\n");
    document.write(n+" "+dt+"\n");
}

function timestring(n) {
    document.write("<pre>\n");
    document.write("DataSet: JavaScript string\n");
    for (i=0;i<n;i++)
        runstring(62500);
    for (i=0;i<n;i++)
        runstring(125000);
    for (i=0;i<n;i++)
        runstring(250000);
    for (i=0;i<n;i++)
        runstring(500000);
    document.write("</pre>\n");
}


//////////////////////////////////////////////////////////////////////////
// timeall
// Run all the tests
//
function timeall(n) {
    timesumloop(n);
    timeackf(n);
    timearray1(n);
    timestring(n);
}
