#! /bin/sh
# @(#)timing.sh	1.3 01/30/98
# Copyright (c) 1998 The Regents of the University of California.
# All rights reserved.
# 
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the
# above copyright notice and the following two paragraphs appear in all
# copies of this software.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
# THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# 
# THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
# CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.
# 
# 						PT_COPYRIGHT_VERSION_2
# 						COPYRIGHTENDKEY

# Test to run Kernighan's Java tests and produce xgraph output

# We define different shell procedures for the different languages,
# and then loop throught them, running all the tests for each language.
#
# Usage: timing.sh timingtest_directory {lang}
# Where lang is optional and is used to chose a language.  If
# the lang argument is not present, then all the languages are run

# Define the shell procedures for each test

# Number of times to run each test
count="1 2 3 4 5 6 7 8 9 10"


CLASSDIR=`pwd`/classes

if [ $# -ne 1 ]; then
    echo "$0: Usage: timing.sh timingtest_directory {lang}"
    exit 9
fi

# Location of Brian Kernighan's Timing test sources and datafiles
TIMINGTESTS=$1
shift

# Use tclsh to run Tcl tests
# Usage: tclsh test X
# Run the test, producing x/y pairs, where X is the value
# being computed, and Y is the time.
testtclsh () { 
    for i in $count
    do
        case $1 in 
	    ack)
	        if [ $3 != 5 ]; then
		    echo "# $TCLSH can't handle ack(3,$3)"
		else    
		gtime -f " %e elapsed" $TCLSH $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$3 -
		fi
		;;

	    cat|tail)
		# Don't pass the 3 arg in, since cat.tcl and tail.tcl just
	        # end up reading it
		gtime -f " %e elapsed" $TCLSH $TIMINGTESTS/$1.tcl $2 1>/dev/null 2>/tmp/tclshcat.out
		cat /tmp/tclshcat.out | 
		awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$4 -
		;;
	    sum1)
		gtime -f " %e elapsed" $TCLSH $TIMINGTESTS/$1.tcl $2 2>&1 |
		awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$3 -
		;;
	    wc)
		gtime -f " %e elapsed" $TCLSH $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		awk '{if ($2 !~ /elapsed/) {save=$0;lines=$1} else {print "#"save; print lines, $1}}' -
		;;
	    *)
		gtime -f " %e elapsed" $TCLSH $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$2 -
		;;
	esac
    done
}

# Use Jacl to run Tcl tests
# Usage: jacl test X
# Run the test, producing x/y pairs, where X is the value
# being computed, and Y is the time.
testjacl () { 
    count="1 2 3 4"
    if [ $1 = string ]; then
        # String is fairly fast
	count="1 2 3 4 5 6 7 8 9"
    fi	
    for i in $count
    do
        case $1 in 
	    ack)
    	        if [ $3 != 5 ]; then
		    echo "# jacl can't handle ack(3,$3), message is: too many nested calls to eval (infinite loop?)"
		else    
		    gtime -f " %e elapsed" jacl $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		    awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$3 -
		fi
		;;
	    cat)
		gtime -f " %e elapsed" jacl $TIMINGTESTS/$1.tcl $2 2>&1 |
	        awk '{if ($2 ~ /elapsed/) {print "# ",NR; print x, $1}}' x=$4 -
		;;
	    tail)
	        echo "jacl runs out of memory for tail $4"
		#gtime -f " %e elapsed" jacl $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		#awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$4 -
		;;

	    sum1)
		gtime -f " %e elapsed" jacl $TIMINGTESTS/$1.tcl $2 2>&1 |
		awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$3 -
		;;
	    wc)
		gtime -f " %e elapsed" jacl $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		awk '{if($2 !~ /elapsed/) {save=$0;lines=$1} else {print lines, $1}}' -
		;;
	    *)
		gtime -f " %e elapsed" jacl $TIMINGTESTS/$1.tcl $2 $3 2>&1 |
		awk '{if ($2 !~ /elapsed/) {save=save $0} else {print "#"save; print x, $1}}' x=$2 -
		;;
	esac
    done
}

# Use Java to run Java tests
# Usage: runtest test X
# Run the test, producing x/y pairs, where X is the value
# being computed, and Y is the time.
testjava () { 
    for i in $count
    do
        case $1 in 
	    ack)
		CLASSPATH=$CLASSDIR java $1 $2 $3 2>&1 | 
		awk '{print "#"$0; print x, $2/1000}' x=$3 -
		;;
	    cat)
		CLASSPATH=$CLASSDIR java $1 $2 $3 2>&1 | 
		awk '{print "#"$0; print x,$1/1000}' x=$4 -
		;;
	    sum1)
		CLASSPATH=$CLASSDIR java $1 $2 2>&1 | 
		awk '{print "#"$0; print x,$2/1000}' x=$3 -
		;;
	    tail)
		CLASSPATH=$CLASSDIR java $1 $2  >/dev/null 2>javatail.out
		awk '{print "#"$0; print x, $2/1000}' x=$4 < javatail.out
		;;
	    wc)
		CLASSPATH=$CLASSDIR java $1 $2  | 2>&1 \
		awk '{if ($0 !~ /msec/) {save=$0; lines=$1}  else  {print "#"save; print lines, $1/1000}}'  -
		;;
	    *)
		CLASSPATH=$CLASSDIR java $1 $2  | 2>&1 \
		awk '{print "#"$0; print x, $2/1000}' x=$2 -
		;;
	esac	
    done
}

# Use Tcl Blend to call Java
# Usage: runtest test X
# Run the test, producing x/y pairs, where X is the value
# being computed, and Y is the time.
testjtclsh () { 
    for i in $count
    do
        case $1 in 
	    ack)
		echo "set n [java::new $1]; set v [java::new {String[]} {2} {$2 $3}]; \$n run$1 \$v; exit" | \
		CLASSPATH=$CLASSDIR jtclsh 2>&1 |
		awk '{print "#"$0; print x, $3/1000}' x=$3 -
		;;
	    cat)
		echo "set n [java::new $1]; set v [java::new {String[]} {2} {$2 $3}]; \$n run$1 \$v; exit" | \
		CLASSPATH=$CLASSDIR jtclsh 2>&1 |
		awk '{print "#"$0; print x, $2/1000}' x=$4 -
		;;
	    sum1)
		echo "set n [java::new $1]; set v [java::new {String[]} {2} {$2 $3}]; \$n run$1 \$v; exit" | \
		CLASSPATH=$CLASSDIR jtclsh 2>&1 |
		awk '{print "#"$0; print x,$3/1000}' x=$3 -
		;;
	    tail)
		echo "set n [java::new $1]; set v [java::new {String[]} {1} {$2}]; \$n run$1 \$v; exit" | \
		CLASSPATH=$CLASSDIR jtclsh  >/dev/null 2>jtclshtail.out 
		awk '{print "#"$0; print x, $2/1000}' x=$4 < jtclshtail.out
		;;

	    wc)
		echo "set n [java::new $1]; set v [java::new {String[]} {1} {$2}]; \$n run$1 \$v; exit" | \
		CLASSPATH=$CLASSDIR jtclsh  | 2>&1 \
		awk '{if ($0 !~ /msec/){save=$0; lines=$2} else {print "# "save; print lines, $1/1000}}' -
		;;
	    *)
		echo "set n [java::new $1]; set v [java::new {String[]} {1} {$2}]; \$n run$1 \$v; exit" | \
		CLASSPATH=$CLASSDIR jtclsh  | 2>&1 \
		awk '$0 ~/msec/{print "#"$0; print x, $3/1000}' x=$2 -
		;;
	esac
    done
}

# For each language, run all the tests.

# Process the arguments
if [ $# -gt 0 ]; then
    langs="$1"
else
    langs="jacl tclsh7.6 tclsh jtclsh java"
fi

echo "# `date`"
echo "# Running using the following binaries"
for lang in $langs
do
    echo "# `which $lang`"
done

testjacl array1 50000
for lang in $langs
do
    if [ $lang = tclsh7.6 ]; then
	TCLSH=tclsh7.6
	lang=tclsh
	desc=tclsh7.6
    else
	TCLSH=tclsh
	desc=$lang
    fi	

    echo "DataSet: $desc sumloop"
    for n in 125000 250000 500000 1000000
    do
    	test$lang sumloop $n
    done

    echo "DataSet: $desc ack"
	test$lang ack 3 5
	test$lang ack 3 6
	test$lang ack 3 7
	test$lang ack 3 8

    echo "DataSet: $desc array1"
    for n in 25000 50000 100000 200000
    do
	test$lang array1 $n
    done

    echo "DataSet: $desc string"
    for n in 62500 125000 250000 500000
    do
	test$lang string $n
    done

    echo "DataSet: $desc assoc"
	for n in 5000 10000 20000 40000
    do
	test$lang assoc $n
    done

    # No need to cd, the makefile will create links for us.
    #cd $TIMINGTESTS
    echo "DataSet: $desc cat"
	test$lang cat bib10k foo `wc -l < bib10k`
	test$lang cat bib20k foo `wc -l < bib20k`
	test$lang cat bib foo `wc -l < bib`

    echo "DataSet: $desc wc"
	test$lang wc bib10k
	test$lang wc bib20k
	test$lang wc bib

    echo "DataSet: $desc tail"
	test$lang tail bib10k foo `wc -l < bib10k`
	test$lang tail bib20k foo `wc -l < bib20k`
	test$lang tail bib foo `wc -l < bib`

    echo "DataSet: $desc sum1"
	test$lang sum1 rn25 `wc -l < rn25`
	test$lang sum1 rn50 `wc -l < rn50`
	test$lang sum1 rn100 `wc -l < rn100`

done
